# Generated by Django 3.0.8 on 2020-07-07 17:00

from django.db import migrations

from . import create_device_firmware_for_connections


def create_device_firmware_for_connections_helper(apps, schema_editor):
    app_label = 'firmware_upgrader'
    create_device_firmware_for_connections(apps, schema_editor, app_label)


class Migration(migrations.Migration):
    dependencies = [
        ('firmware_upgrader', '0005_build_os_identifier'),
    ]

    operations = [
        migrations.RunPython(
            create_device_firmware_for_connections_helper,
            reverse_code=migrations.RunPython.noop,
        ),
    ]
