# Generated by Django 3.2.18 on 2023-03-03 21:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('firmware_upgrader', '0008_firmware_image_baseurl'),
    ]

    operations = [
        migrations.AddField(
            model_name='batchupgradeoperation',
            name='upgrade_options',
            field=models.JSONField(default=dict, blank=True),
        ),
        migrations.AddField(
            model_name='upgradeoperation',
            name='upgrade_options',
            field=models.JSONField(default=dict, blank=True),
        ),
    ]
