# Generated by Django 3.1.5 on 2021-01-21 16:57

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('firmware_upgrader', '0006_create_device_firmware'),
    ]

    operations = [
        migrations.AlterField(
            model_name='upgradeoperation',
            name='image',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.FIRMWARE_UPGRADER_FIRMWAREIMAGE_MODEL,
            ),
        ),
    ]
