# Generated by Django 3.0.8 on 2020-07-07 17:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('firmware_upgrader', '0004_batch_upgrade_operation_idle_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='build',
            name='os',
            field=models.CharField(
                blank=True,
                help_text='OS identifier as presented by the device, used to automatically recognize the firmware image used by new devices that register into the system',
                max_length=64,
                null=True,
                verbose_name='OS identifier',
            ),
        ),
    ]
