#id_devicefirmware-0-upgrade_options_jsoneditor {
  padding: 15px;
  border-bottom: 1px solid var(--hairline-color);
}
#id_devicefirmware-0-upgrade_options_jsoneditor label {
  width: unset;
  padding: 0px;
}
#id_devicefirmware-0-upgrade_options_jsoneditor input[type="checkbox"] {
  vertical-align: middle;
}
#main #id_devicefirmware-0-upgrade_options_jsoneditor .form-row {
  padding: 0px;
  border-bottom: none;
}
#devicefirmware-group .form-row.field-upgrade_options {
  display: none;
}
